@@ -13,11 +13,21 @@ layer_sf <- function(geom = NULL, stat = NULL,
                      position = NULL, params = list(),
                      inherit.aes = TRUE, check.aes = TRUE, check.param = TRUE,
                      show.legend = NA) {
+  if (is.character(show.legend)) {
+    legend_key_type <- show.legend
+    show.legend <- TRUE
+  } else {
+    legend_key_type <- NULL
+  }
+
+  # inherit from LayerSf class to add `legend_key_type` slot
+  layer_class <- ggproto(NULL, LayerSf, legend_key_type = legend_key_type)
+
   layer(
     geom = geom, stat = stat, data = data, mapping = mapping,
     position = position, params = params, inherit.aes = inherit.aes,
     check.aes = check.aes, check.param = check.param,
-    show.legend = show.legend, layer_class = LayerSf
+    show.legend = show.legend, layer_class = layer_class
   )
 }
 
@@ -32,25 +42,29 @@ LayerSf <- ggproto("LayerSf", Layer,
         (!isTRUE(self$inherit.aes) && is.null(self$mapping$geometry))) {
       if (is_sf(data)) {
         geometry_col <- attr(data, "sf_column")
-        self$mapping$geometry <- as.name(geometry_col)
+        self$mapping$geometry <- sym(geometry_col)
       }
     }
 
     # automatically determine the legend type
-    if (is.na(self$show.legend) || isTRUE(self$show.legend)) {
-      if (is_sf(data)) {
-        sf_type <- detect_sf_type(data)
-        if (sf_type == "point") {
-          self$geom_params$legend <- "point"
-        } else if (sf_type == "line") {
-          self$geom_params$legend <- "line"
-        } else {
-          self$geom_params$legend <- "polygon"
+    if (is.null(self$legend_key_type)) {
+      # first, set default value in case downstream tests fail
+      self$geom_params$legend <- "polygon"
+
+      # now check if the type should not be polygon
+      if (!is.null(self$mapping$geometry)) {
+        geometry_column <- as_name(self$mapping$geometry)
+        if (inherits(data[[geometry_column]], "sfc")) {
+          sf_type <- detect_sf_type(data[[geometry_column]])
+          if (sf_type == "point") {
+            self$geom_params$legend <- "point"
+          } else if (sf_type == "line") {
+            self$geom_params$legend <- "line"
+          }
         }
       }
-    } else if (is.character(self$show.legend)) {
-      self$geom_params$legend <- self$show.legend
-      self$show.legend <- TRUE
+    } else {
+      self$geom_params$legend <- self$legend_key_type
     }
     data
   }
