@@ -2,10 +2,12 @@
 #'
 #' Default line types based on a set supplied by Richard Pearson,
 #' University of Manchester. Continuous values can not be mapped to
-#' line types.
+#' line types unless `scale_linetype_binned()` is used. Still, as linetypes has
+#' no inherent order, this use is not advised.
 #'
 #' @inheritParams scale_x_discrete
-#' @param na.value The linetype to use for \code{NA} values.
+#' @inheritDotParams discrete_scale -expand -position -na.value
+#' @param na.value The linetype to use for `NA` values.
 #' @rdname scale_linetype
 #' @export
 #' @examples
@@ -32,10 +34,16 @@ scale_linetype <- function(..., na.value = "blank") {
     na.value = na.value, ...)
 }
 
+#' @rdname scale_linetype
+#' @export
+scale_linetype_binned <- function(..., na.value = "blank") {
+  binned_scale("linetype", "linetype_b", binned_pal(linetype_pal()), ...)
+}
+
 #' @rdname scale_linetype
 #' @export
 scale_linetype_continuous <- function(...) {
-  stop("A continuous variable can not be mapped to linetype", call. = FALSE)
+  abort("A continuous variable can not be mapped to linetype")
 }
 #' @rdname scale_linetype
 #' @export
