@@ -278,18 +278,23 @@ snake_class <- function(x) {
 }
 
 empty <- function(df) {
-  is.null(df) || nrow(df) == 0 || ncol(df) == 0
+  is.null(df) || nrow(df) == 0 || ncol(df) == 0 || is.waive(df)
 }
 
 is.discrete <- function(x) {
   is.factor(x) || is.character(x) || is.logical(x)
 }
 
-# This function checks that all columns of a dataframe `x` are data and
-# returns the names of any columns that are not.
-# We define "data" as atomic types or lists, not functions or otherwise
+# This function checks that all columns of a dataframe `x` are data and returns
+# the names of any columns that are not.
+# We define "data" as atomic types or lists, not functions or otherwise.
+# The `inherits(x, "Vector")` check is for checking S4 classes from Bioconductor
+# and wether they can be expected to follow behavior typical of vectors. See
+# also #3835
 check_nondata_cols <- function(x) {
-  idx <- (vapply(x, function(x) is.null(x) || rlang::is_vector(x), logical(1)))
+  idx <- (vapply(x, function(x) {
+    is.null(x) || rlang::is_vector(x) || inherits(x, "Vector")
+  }, logical(1)))
   names(x)[which(!idx)]
 }
 
@@ -379,7 +384,7 @@ NULL
 # Check inputs with tibble but allow column vectors (see #2609 and #2374)
 as_gg_data_frame <- function(x) {
   x <- lapply(x, validate_column_vec)
-  new_data_frame(tibble::as_tibble(x))
+  new_data_frame(x)
 }
 validate_column_vec <- function(x) {
   if (is_column_vec(x)) {
@@ -430,11 +435,11 @@ switch_orientation <- function(aesthetics) {
   aesthetics
 }
 
-#' Utilities for working with bidirecitonal layers
+#' Utilities for working with bidirectional layers
 #'
 #' These functions are what underpins the ability of certain geoms to work
-#' automatically in both directions. See the *Extending ggplot2* for how they
-#' are used when implementing `Geom`, `Stat`, and `Position` classes.
+#' automatically in both directions. See the *Extending ggplot2* vignette for
+#' how they are used when implementing `Geom`, `Stat`, and `Position` classes.
 #'
 #' `has_flipped_aes()` is used to sniff out the orientation of the layer from
 #' the data. It has a range of arguments that can be used to finetune the
@@ -456,9 +461,7 @@ switch_orientation <- function(aesthetics) {
 #' - `range_is_orthogonal`: This argument controls whether the existance of
 #'   range-like aesthetics (e.g. `xmin` and `xmax`) represents the main or
 #'   secondary axis. If `TRUE` then the range is given for the secondary axis as
-#'   seen in e.g. [geom_ribbon()] and [geom_linerange()]. `FALSE` is less
-#'   prevalent but can be seen in [geom_bar()] where it may encode the span of
-#'   each bar.
+#'   seen in e.g. [geom_ribbon()] and [geom_linerange()].
 #' - `group_has_equal`: This argument controls whether to test for equality of
 #'   all `x` and `y` values inside each group and set the main axis to the one
 #'   where all is equal. This test is only performed if `TRUE`, and only after
@@ -559,10 +562,10 @@ has_flipped_aes <- function(data, params = list(), main_is_orthogonal = NA,
   }
 
   # Is there a single actual discrete position
-  y_is_int <- is.integer(y)
-  x_is_int <- is.integer(x)
-  if (xor(y_is_int, x_is_int)) {
-    return(y_is_int != main_is_continuous)
+  y_is_discrete <- is_mapped_discrete(y)
+  x_is_discrete <- is_mapped_discrete(x)
+  if (xor(y_is_discrete, x_is_discrete)) {
+    return(y_is_discrete != main_is_continuous)
   }
 
   # Does each group have a single x or y value
@@ -583,51 +586,6 @@ has_flipped_aes <- function(data, params = list(), main_is_orthogonal = NA,
     }
   }
 
-  # give up early
-  if (!has_x && !has_y) {
-    return(FALSE)
-  }
-
-  # Both true discrete. give up
-  if (y_is_int && x_is_int) {
-    return(FALSE)
-  }
-  # Is there a single discrete-like position
-  y_is_int <- if (has_y) isTRUE(all.equal(y, round(y))) else FALSE
-  x_is_int <- if (has_x) isTRUE(all.equal(x, round(x))) else FALSE
-  if (xor(y_is_int, x_is_int)) {
-    return(y_is_int != main_is_continuous)
-  }
-
-  if (main_is_optional) {
-    # Is one of the axes all 0
-    if (all(x == 0)) {
-      return(main_is_continuous)
-    }
-    if (all(y == 0)) {
-      return(!main_is_continuous)
-    }
-  }
-
-  y_diff <- diff(sort(y))
-  x_diff <- diff(sort(x))
-
-  # FIXME: If both are discrete like, give up. Probably, we can make a better
-  # guess, but it's not possible with the current implementation as very little
-  # information is available in Geom$setup_params().
-  if (y_is_int && x_is_int) {
-    return(FALSE)
-  }
-
-  y_diff <- y_diff[y_diff != 0]
-  x_diff <- x_diff[x_diff != 0]
-
-  # If none are discrete is either regularly spaced
-  y_is_regular <- if (has_y && length(y_diff) != 0) all((y_diff / min(y_diff)) %% 1 < .Machine$double.eps) else FALSE
-  x_is_regular <- if (has_x && length(x_diff) != 0) all((x_diff / min(x_diff)) %% 1 < .Machine$double.eps) else FALSE
-  if (xor(y_is_regular, x_is_regular)) {
-    return(y_is_regular != main_is_continuous)
-  }
   # default to no
   FALSE
 }
@@ -653,3 +611,10 @@ flipped_names <- function(flip = FALSE) {
   names(ret) <- c(x_aes, y_aes)
   ret
 }
+
+split_with_index <- function(x, f, n = max(f)) {
+  if (n == 1) return(list(x))
+  f <- as.integer(f)
+  attributes(f) <- list(levels = as.character(seq_len(n)), class = "factor")
+  unname(split(x, f))
+}
